@@ -90,8 +90,9 @@ fn get_ast(tokens: &[Token]) -> Result<Expression, Error> {
     _get_ast(tokens, &mut pos)
 }
 
-/// Parses the input lambda expression to a `Term`; the lambda can be represented either with the
-/// greek letter (λ) or a backslash (\\ - less aesthetic, but only one byte in size).
+/// Parses the input `&str` as a lambda `Term`. The lambdas can be represented either with the
+/// greek letter (λ) or a backslash (\\ - less aesthetic, but only one byte in size), the De Bruijn
+/// indices start with 1 and are hexadecimal digits, and whitespaces are ignored.
 ///
 /// # Example
 /// ```
@@ -204,19 +205,48 @@ mod test {
     #[test]
     fn parse_y() {
         let y = "λ(λ2(11))(λ2(11))";
-        assert_eq!(&*format!("{}", parse(&y).expect("parsing Y failed!")), y);
+        assert_eq!(parse(&y).expect("parsing Y failed!"),
+            abs(
+                app(
+                    abs(
+                        app(
+                            Var(2),
+                            app(Var(1), Var(1))
+                        )
+                    ),
+                    abs(
+                        app(
+                            Var(2),
+                            app(Var(1), Var(1))
+                        )
+                    )
+                )
+            )
+        );
     }
 
     #[test]
     fn parse_quine() {
         let quine = "λ1((λ11)(λλλλλ14(3(55)2)))1";
-        assert_eq!(&*format!("{}", parse(&quine).expect("parsing QUINE failed!")), quine);
+        assert_eq!(parse(&quine).expect("parsing QUINE failed!"),
+            abs(app(app(Var(1), app(abs(app(Var(1), Var(1))), abs(abs(abs(abs(abs(app(app(Var(1),
+            Var(4)), app(app(Var(3), app(Var(5), Var(5))), Var(2)))))))))), Var(1)))
+        );
     }
 
     #[test]
     fn parse_blc() {
         let blc = "(λ11)(λλλ1(λλλλ3(λ5(3(λ2(3(λλ3(λ123)))(4(λ4(λ31(21))))))(1(2(λ12))\
                    (λ4(λ4(λ2(14)))5))))(33)2)(λ1((λ11)(λ11)))";
-        assert_eq!(&*format!("{}", parse(&blc).expect("parsing BLC failed!")), blc);
+        assert_eq!(parse(&blc).expect("parsing BLC failed!"),
+            app(app(abs(app(Var(1), Var(1))), abs(abs(abs(app(app(app(Var(1),
+            abs(abs(abs(abs(app(Var(3), abs(app(app(Var(5), app(Var(3), abs(app(app(Var(2),
+            app(Var(3), abs(abs(app(Var(3), abs(app(app(Var(1), Var(2)), Var(3)))))))), app(Var(4),
+            abs(app(Var(4), abs(app(app(Var(3), Var(1)), app(Var(2), Var(1))))))))))),
+            app(app(Var(1), app(Var(2), abs(app(Var(1), Var(2))))), abs(app(app(Var(4),
+            abs(app(Var(4), abs(app(Var(2), app(Var(1), Var(4))))))), Var(5)))))))))))),
+            app(Var(3), Var(3))), Var(2)))))), abs(app(Var(1), app(abs(app(Var(1), Var(1))),
+            abs(app(Var(1), Var(1)))))))
+        );
     }
 }
