@@ -2,6 +2,21 @@
 
 use term::*;
 use term::Term::*;
+use self::Order::*;
+
+/// Set to `true` to see all the steps of β-reductions. The default is `false`.
+pub const SHOW_REDUCTIONS: bool = false;
+
+/// The [evaluation order](https://en.wikipedia.org/wiki/Lambda_calculus#Reduction strategies) of 
+/// β-reductions. `Applicative` order will not fully reduce expressions containing functions 
+/// without a normal form, e.g. the Y combinator. The default is `Normal`.
+pub const EVALUATION_ORDER: Order = Normal;
+
+#[derive(Debug, PartialEq)]
+pub enum Order {
+    Normal,
+    Applicative
+}
 
 /// Applies two terms with substitution and variable update, consuming the first one in the process.
 ///
@@ -55,9 +70,6 @@ fn update_free_variables(term: &mut Term, added_depth: usize, own_depth: usize)
     }
 }
 
-/// Set to `true` to see all the steps of β-reductions. The default is `false`.
-pub const SHOW_REDUCTIONS: bool = false;
-
 impl Term {
     /// Applies `self` to another term with substitution and variable update, consuming it
     /// in the process.
@@ -93,7 +105,7 @@ impl Term {
         apply(lhs, &rhs)
     }
 
-    /// Performs a single normal-order β-reduction on `self`.
+    /// Performs a single β-reduction on `self`.
     ///
     /// # Example
     ///
@@ -106,32 +118,88 @@ impl Term {
     /// assert_eq!(succ_one, succ().apply(&1.into()).unwrap());
     /// ```
     pub fn beta_once(&mut self) {
-        let mut done = false;
-        self._beta_once(&mut done, 0);
+        match EVALUATION_ORDER {
+            Normal => self._beta_once_normal(0),
+            Applicative => self._beta_once_applicative(0)
+        };
     }
 
-    fn _beta_once(&mut self, done: &mut bool, depth: u32) {
-        if *done { return }
-
+    fn _beta_once_normal(&mut self, depth: u32) -> bool {
         match *self {
-            Var(_) => (),
-            Abs(_) => self.unabs_ref_mut().unwrap()._beta_once(done, depth + 1),
+            Var(_) => false,
+            Abs(_) => self.unabs_ref_mut().unwrap()._beta_once_normal(depth + 1),
             App(_, _) => {
                 if self.lhs_ref().unwrap().unabs_ref().is_ok() {
                     let copy = self.clone();
                     if SHOW_REDUCTIONS { print!("    {} reduces to ", show_precedence(self, 0, depth)) };
                     *self = copy.eval().unwrap();
-                    if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth + 1)) }
-                    *done = true;
+                    if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) }
+                    true
                 } else {
-                    self.lhs_ref_mut().unwrap()._beta_once(done, depth);
-                    self.rhs_ref_mut().unwrap()._beta_once(done, depth)
+                    if !self.lhs_ref_mut().unwrap()._beta_once_normal(depth) {
+                        self.rhs_ref_mut().unwrap()._beta_once_normal(depth)
+                    } else {
+                        true
+                    }
+                }
+            }
+        }
+    }
+    
+    fn _beta_once_applicative(&mut self, depth: u32) -> bool {
+        match *self {
+            Var(_) => false,
+            Abs(_) => self.unabs_ref_mut().unwrap()._beta_once_applicative(depth + 1),
+            App(_, _) => {
+                if !self.lhs_ref().unwrap().is_beta_reducible() &&
+                   !self.rhs_ref().unwrap().is_beta_reducible() &&
+                    self.lhs_ref().unwrap().unabs_ref().is_ok()
+                {
+                    let copy = self.clone();
+                    if SHOW_REDUCTIONS { print!("    {} reduces to ", show_precedence(self, 0, depth)) };
+                    *self = copy.eval().unwrap();
+                    if SHOW_REDUCTIONS { println!("{}", show_precedence(self, 0, depth)) }
+                    true
+                } else {
+                    if !self.lhs_ref_mut().unwrap()._beta_once_applicative(depth) {
+                        self.rhs_ref_mut().unwrap()._beta_once_applicative(depth)
+                    } else {
+                        true
+                    }
+                }
+            }
+        }
+    }
+    
+    /// Checks whether `self` is β-reducible.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lambda_calculus::arithmetic::{succ, zero};
+    ///
+    /// let reducible     = succ().app(1.into());
+    /// let not_reducible = zero();
+    ///
+    /// assert!(reducible.is_beta_reducible());
+    /// assert!(!not_reducible.is_beta_reducible());
+    /// ```
+    pub fn is_beta_reducible(&self) -> bool {
+        match *self {
+            Var(_) => false,
+            Abs(_) => self.unabs_ref().unwrap().is_beta_reducible(),
+            App(_, _) => {
+                if self.lhs_ref().unwrap().unabs_ref().is_ok() {
+                    true
+                } else {
+                    self.lhs_ref().unwrap().is_beta_reducible() ||
+                    self.rhs_ref().unwrap().is_beta_reducible()
                 }
             }
         }
     }
 
-    /// Performs full normal-order β-reduction on `self`.
+    /// Performs full β-reduction on `self`.
     ///
     /// # Example
     ///
@@ -156,7 +224,7 @@ impl Term {
     }
 }
 
-/// Performs full normal-order β-reduction on a `Term`, consuming it in the process.
+/// Performs full β-reduction on a `Term`, consuming it in the process.
 ///
 /// # Example
 ///
@@ -173,7 +241,7 @@ pub fn beta_full(mut term: Term) -> Term {
     term
 }
 
-/// Performs a single normal-order β-reduction on `self`.
+/// Performs a single β-reduction on `self`.
 ///
 /// # Example
 ///
@@ -192,5 +260,22 @@ pub fn beta_once(mut term: Term) -> Term {
 
 #[cfg(test)]
 mod test {
-
+    use super::*;
+    use parser::parse;
+    
+    #[test]
+    fn normal_order() {
+        if EVALUATION_ORDER == Normal {
+            let should_reduce = parse(&"(λ2)((λ111)(λ111))").unwrap();
+            assert_eq!(beta_full(should_reduce), Var(1))
+        }
+    }
+    
+    #[test]
+    fn applicative_order() {
+        if EVALUATION_ORDER == Applicative {
+            let shouldnt_reduce = parse(&"(λ2)((λ111)(λ111))").unwrap();
+            assert_eq!(beta_once(shouldnt_reduce), parse(&"(λ2)((λ111)(λ111)(λ111))").unwrap())
+        }
+    }
 }
